/**
 * Copyright 2019-2021 Britanicus <marcusbritanicus@gmail.com>
 *
 * This file is a part of DesQ Shell (https://gitlab.com/DesQ/Shell)
 * DesQ Shell is the main Shell UI of the DesQ Project
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * at your option, any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301, USA.
 **/

// Local Headers
#include "DesQContainer.hpp"

// LibDesQ Headers
#include <desq/Utils.hpp>
#include <desqui/Application.hpp>

// Wayland Headers
#include <wlrootsqt/WlGlobal.hpp>
#include <wlrootsqt/Registry.hpp>
#include <wlrootsqt/LayerShell.hpp>
#include <wlrootsqt/Application.hpp>

WlrootsQt::LayerSurface *cls;

void raiseWidgets() {
    if ( cls ) {
        cls->setLayer( WlrootsQt::LayerShell::Overlay );
        cls->apply();
    }
}


void lowerWidgets() {
    if ( cls ) {
        cls->setLayer( WlrootsQt::LayerShell::Bottom );
        cls->apply();
    }
}


int main( int argc, char **argv ) {
    qInstallMessageHandler( DesQ::Logger );
    qputenv( "QT_WAYLAND_USE_BYPASSWINDOWMANAGERHINT", QByteArrayLiteral( "1" ) );

    QCoreApplication::setAttribute( Qt::AA_EnableHighDpiScaling );

    WlrootsQt::Application app( "DesQContainer", argc, argv );

    if ( app.isRunning() ) {
        if ( qApp->arguments().contains( "--raise" ) ) {
            app.sendMessage( "raise" );
        }

        else if ( qApp->arguments().contains( "--lower" ) ) {
            app.sendMessage( "lower" );
        }

        return 0;
    }

    QCommandLineParser parser;

    parser.addHelpOption();         // Help
    parser.addVersionOption();      // Version

    /* Widgets to be loaded */
    parser.addPositionalArgument( "widgets", "Widgets to be loaded", "widgets" );

    /* Process the CLI args */
    parser.process( app );

    DesQContainer *widget;

    if ( parser.positionalArguments().count() ) {
        widget = new DesQContainer( parser.positionalArguments() );
    }

    else {
        printf( "DesQ Containder v1.0.0\n" );
        parser.showHelp();
        return 0;
    }

    widget->show();

    if ( WlrootsQt::isRunning() ) {
        Wayland::Registry *registry = qApp->waylandRegistry();

        cls = registry->layerShell()->getLayerSurface( widget->windowHandle(), nullptr, WlrootsQt::LayerShell::Bottom, "desq-widget" );
        cls->setAnchors(
            WlrootsQt::LayerSurface::Top |
            WlrootsQt::LayerSurface::Bottom |
            WlrootsQt::LayerSurface::Right
        );
        cls->setSurfaceSize( widget->size() );
        cls->setMargins( QMargins( 0, 10, 10, 10 ) );
        cls->setExclusiveZone( 0 );
        cls->setKeyboardInteractivity( WlrootsQt::LayerSurface::NoFocus );
        cls->apply();
    }

    QObject::connect(
        &app, &WlrootsQt::Application::messageReceived, [ = ] ( const QString& mesg ) {
            if ( mesg == "raise" ) {
                raiseWidgets();
            }

            else if ( mesg == "lower" ) {
                lowerWidgets();
            }
        }
    );

    return app.exec();
}
